import { useMutation, useQueryClient } from "react-query";
import { useAuth } from "../../state/external/auth-provider";
import { queryKeys } from "../query-keys";
import { AddLandmarkData, Landmark } from "../../types";
import { LOGGING } from "../../utils/logging";

const landmarkMissingProps = (lm: Landmark) => {
    const missingProps = []

    Object.keys(lm).forEach(key => {
        if (!lm[key]) 
            missingProps.push(key);
    });

    return missingProps
}

export const useAddLandmark = () => {
    const {sendApiRequestAsync, userId, anonUserId} = useAuth()
    const queryClient = useQueryClient();

    const createLandmark = async (data: AddLandmarkData): Promise<Landmark | undefined> => {
        if(!data.landmark) {
            LOGGING.log('ADD-LANDMARK', 'error', 'Landmark data is undefined');
            return;
        }

        const missingProps = landmarkMissingProps(data.landmark);
        if (missingProps.length > 0) {
            LOGGING.log('ADD-LANDMARK', 'error', 'Landmark data is missing props: ' + missingProps.join(','));
            return;
        }
            

        // attach landmark owner's id
        if (userId) {
            data.landmark.user = userId
        } else if (anonUserId) {
            data.landmark.anonymous = anonUserId
        }
        else {
            console.warn("[LandmarkData]: Couldn't create landmark, user id or anon id wasn't given.")
            return
        }

        console.log(data)

        const response = await sendApiRequestAsync({
            axiosConfig: {
                method: 'POST',
                url: `/api/landmark/`,
                data: {
                    landmark: data.landmark,
                    photos: data.photos,
                    indoorLmLocImg: data.indoorMapSnapshotBase64
                },
            },
            authorized: true,
            errorMessage: "Something went wrong when creating a landmark",
            loggingCategory: "LANDMARKS"
        });   
        return response?.data;
        
    }

    return useMutation(createLandmark, {
        onSuccess: landmark => {
            queryClient.invalidateQueries(queryKeys.GET_LANDMARKS)
        },  
    })
}