import { FontAwesome } from "@expo/vector-icons"
import { useNavigation, useNavigationState, useRoute } from "@react-navigation/native"
import { createNativeStackNavigator } from "@react-navigation/native-stack"
import { StackNavigationProp } from "@react-navigation/stack"
import { observer } from "mobx-react"
import React, { useState } from "react"
import { Image, Linking, ScrollView, StyleProp, StyleSheet, Text, View, ViewStyle } from "react-native"
import { Menu, MenuDivider, MenuItem } from 'react-native-material-menu'
import { Chip } from "react-native-paper"
import { store } from "../../main-store"
import { useAuth } from "../../state/external/auth-provider"
import { colors, lmTypes } from "../../utils/GlobalUtils"
import { IconButton } from "../Buttons"
import { MainTabsNavigationProp } from "../navigation/main-tabs-navigator"
import { navigate } from "../navigation/root-navigator"
import IndoorMap from "./indoor/indoor-map.view"
import mapStyles from "./Map.styles"
import OutdoorMap, { AuthTabsMapRouteProp } from "./outdoor/outdoor-map.view"
import AddLandmarkPanel from "./panels/add-landmark-panel/add-landmark-panel.view"
import { FilterPanel } from "./panels/filter-panel/filter-panel"
import LandmarkDetails from "./panels/selected-landmark-panel/select-landmark-panel.view"


export type MapStackParamList = {
    Outdoor: { selectedLandmark: string, selectedLandmarks: string[] },
    Indoor: React.FC,
}

const MapStackNavigator = createNativeStackNavigator()

export type MapStackNavigationProp = StackNavigationProp<MapStackParamList>

interface FilterChipProps {
    content: string;
    onClose: () => void;
    visible: boolean;
    avatar: JSX.Element;
    style: StyleProp<ViewStyle>
}

const FilterChip: React.FC<FilterChipProps> = ({onClose, content, visible, avatar}) => {
    return (
        <>
            {visible && 
            <Chip 
                avatar={avatar} 
                style={styles.filterChip} 
                onClose={() => onClose()}>
                {content}
            </Chip>}
        </>
    )
}

const LandmarkTypeFilterChips = () => {
    const filteredTypes = store.filters.current['landmark-types'] as number[];
    const removeLandmarkTypeFilter = (landmarkType: number) => {
        store.filters.setFilters([{
            type: 'landmark-types', 
            value: (store.filters.current["landmark-types"] as number[]).filter(t => t !== landmarkType)
        }])
    }

    return (store.filters.current['landmark-types'] as number[]).map((type, i) => {
        return (
            <FilterChip 
                key={i} 
                avatar={(<Image style={{ height: 22, width: 17 }} source={lmTypes[type].image} />)} 
                style={styles.landmarkTypeFilterChip} 
                onClose={() => removeLandmarkTypeFilter(type)} 
                content={lmTypes[type].label}
                visible={filteredTypes.length > 0}/>
        )
    })
}

const FilterChips: React.FC = () => {

    return (
        <ScrollView horizontal={true} contentContainerStyle={{ alignItems: 'center' }} style={{ marginHorizontal: 10, flexDirection: 'row' }}>
            {/* Min rating filter chip */}
            <FilterChip 
                visible={store.filters.current["min-rating"] > 0} 
                content={"Min rating: " + store.filters.current['min-rating']}
                onClose={() => store.filters.setFilters([{type: 'min-rating', value: 0}])}
                avatar={(<FontAwesome name="star" size={20} color='gray' style={styles.filterChipIcon} />)}
                style={styles.filterChip}/>

            
            <FilterChip 
                visible={store.filters.current['only-owned'] as boolean} 
                content={"My landmarks"}
                onClose={() => store.filters.setFilters([{type: 'only-owned', value: false}])}
                avatar={(<FontAwesome name="user" size={20} color='gray' style={styles.filterChipIcon} />)}
                style={styles.filterChip}/>

            {LandmarkTypeFilterChips()}
        </ScrollView>
    )
}



const MapNavigator: React.FC = () => {
    const { landmarkOwnedByUser } = useAuth()
    const authNavigation = useNavigation() as MainTabsNavigationProp
    const authRoute = useRoute() as AuthTabsMapRouteProp
    const navigationState = useNavigationState(state => state)
    const [visible, setVisible] = useState(false);

    // TODO: apply current filters when landmark data gets updated
    
    return (
        <View style={{ flex: 1 }}>
            <MapStackNavigator.Navigator screenOptions={{ headerShown: false }} initialRouteName="Outdoor">
                <MapStackNavigator.Screen name="Outdoor" component={OutdoorMap} />
                <MapStackNavigator.Screen name="Indoor" component={IndoorMap}/>
            </MapStackNavigator.Navigator>



            {/* Filter chips and button*/}
            {store.filters.panelVisible ?
                <View style={{ top: 60, right: 7.5, position: 'absolute', flexDirection: "row-reverse", justifyContent: 'flex-end' }}>
                    <IconButton size={16} color={colors.red} style={[mapStyles.filterButtonIndoor]} icon="filter" onPress={() => store.filters.panelVisible} />
                    <FilterChips />
                </View>
                :
                <View style={{ top: 10, marginLeft: 40, marginRight: 20, position: 'absolute', flexDirection: "row-reverse", justifyContent: 'flex-end' }}>
                    <IconButton size={20} color={colors.red} style={[mapStyles.filterButtonOutdoor]} icon="filter" onPress={() => store.filters.openPanel()} />
                    
                </View>
            }

            <AddLandmarkPanel/>
            <LandmarkDetails/>
            <FilterPanel
                visible={mapState.filterVisible}
                toggleOnlyOwned={mapState.toggleOnlyOwned}
                onlyOwned={mapState.onlyOwned} toggleFilter={mapState.toggleFilter}
                setMinLmRating={mapState.setMinLmRating}
                setLmFilteredTypes={mapState.setLmTypeFilter}
                lmFilteredTypes={mapState.lmFilteredTypes}
                minLmRating={mapState.minLmRating} />
        </View>
    )
}

const styles = StyleSheet.create({
    filterChipIcon: { textAlign: 'center', textAlignVertical: 'center' },
    filterChip: { borderWidth: 1, borderColor: 'lightgray', marginRight: 5, marginLeft: 10 },
    landmarkTypeFilterChip: { borderWidth: 1, borderColor: 'lightgray', marginHorizontal: 5, justifyContent: 'center' }
})

export default observer(MapNavigator)
