import { useNavigation, useNavigationState } from "@react-navigation/native";
import { booleanPointInPolygon, circle } from "@turf/turf";
import { useCallback, useEffect, useRef, useState } from "react";
import MapView, { LatLng } from "react-native-maps";
import { openSettings } from "react-native-permissions";
import * as Spokestack from 'react-native-spokestack';
import { useLandmarks } from "../../../api/landmarks";
import { store } from "../../../main-store";
import { usePermissions } from "../../../permissions-context";
import { useAuth } from "../../../state/external/auth-provider";
import { MapStackNavigationProp } from "../map-navigator";

export interface UserLocation {
    latitude: number;
    longitude: number;
    heading?: number;
}

export const useOutdoorMap = () => {
    const {locationPermissionsGranted, voicePermissionsGranted} = usePermissions();
    const {setAlert} = useAuth();

    const [loading, setLoading] = useState<boolean>(false);
    const [userLocation, setUserLocation] = useState<LatLng>();
    const [followUser, setFollowUser] = useState<boolean>();

    const {data: landmarks} = useLandmarks();
    const fromIndoors = useRef<boolean>();
    const mapRef = useRef<MapView>();
    const mapNavigation = useNavigation<MapStackNavigationProp>();

    // used to determine previous screens and decide if the map should be refreshed
    const {index: mapNavigationIndex} = useNavigationState(state => state)

    // when either the map navigation state or main navigation state changes, check to see if we are coming from indoor map. if so, refresh the map to prevent map lag
    useEffect(() => {
        if (fromIndoors.current) {
            fromIndoors.current = false
            setLoading(true)
        }

        setTimeout(() => {
            setLoading(false)
        }, 500);

    }, [mapNavigationIndex])

    // Toggle the lm details panel when a new selected landmark is detected (triggered by pressing on a map marker, or from the list of nearby landmarks)
    useEffect(() => {
        console.log("[LandmarkDetails]: Landmark selected - " + store.selectedLm.selectedLandmarkId)
        if (store.selectedLm.selectedLandmarkId) {
            const landmark = landmarks.find(lm => lm.id == store.selectedLm.selectedLandmarkId)
            mapRef.current.animateToRegion({ latitude: landmark.latitude, longitude: landmark.longitude, latitudeDelta: 0.01, longitudeDelta: 0.01 })
        }
    }, [store.selectedLm.selectedLandmarkId])

    
    // Move to pressed location when newlandmark changes
    useEffect(() => {
        if (store.selectedLm.selectedLandmarkId) {
            mapRef.current.animateToRegion({ 
                latitude: store.addLm.pendingLandmark?.latitude, 
                longitude: store.addLm.pendingLandmark?.longitude, 
                latitudeDelta: 0.01, 
                longitudeDelta: 0.01 
            })
        }
    }, [store.addLm.pendingLandmark])

    const goIndoors = () => useCallback(() => {
        mapNavigation.navigate("Indoor");
    }, [mapNavigation])

    /**
     * Animates the map to fly over to and focus on the user's location.
     */
    const flyToUser = () => useCallback(() => {
        console.log('[Map]: Centering on user')
        setFollowUser(true);
        if (userLocation) {
            mapRef.current?.animateToRegion({ 
                latitude: userLocation.latitude, 
                longitude: userLocation.longitude, 
                latitudeDelta: 0.01, 
                longitudeDelta: 0.01 
            })
        }
    }, [userLocation])

    /**
     * Activates speech recognition and opens the voice panel
     */
    const startSpeech = async () => {
        if (!(voicePermissionsGranted && locationPermissionsGranted)) {
            setAlert({
                title: "Permissions",
                message: "Please enable location and voice permissions in the settings menu",
                type: "warning",
                callback: () => openSettings(),
                callbackButtonText: 'Open settings'
            })
            return
        }
        store.selectedLm.close();
        store.addLm.close();
        const spokestackInitialized = await Spokestack.isInitialized()
        const spokestackStarted = await Spokestack.isStarted()

        if (spokestackInitialized && spokestackStarted) {
            store.addLm.stageNewLandmarkWithLocation({latitude: userLocation.latitude, longitude: userLocation.longitude})
            store.voice.toggleVisible(true)
            Spokestack.activate()
        }
    }

    /**
     * Gets initial region that map should zoom into from current user location
     */
    const getInitialRegion = () => {
        if (userLocation) {
            return { latitude: userLocation.latitude, longitude: userLocation.longitude, latitudeDelta: 0.01, longitudeDelta: 0.01 }
        }
    }

    const onLocationUpdate = async (newCoords: LatLng) => {
        setUserLocation(newCoords)
        // get 10m radius around user
        const userAlertRadius = circle([newCoords.longitude, newCoords.latitude], 10, { units: 'meters' })

        // check each landmark to see if its inside user radius. if it is, and it isn't already in the list of notified landmarks, add it
        const newLandmarksNearUser = landmarks?.filter(lm => {
            const landmarkNearUser = booleanPointInPolygon([lm.longitude, lm.latitude], userAlertRadius)
            return landmarkNearUser
        })

        // to prevent duplicate notifications make a list of landmarks that weren't previously near the user. 
        // these are the only ones that the user will be notified of
        const newLandmarksNotPreviouslyNearUser = newLandmarksNearUser?.filter(lm => store.nearbyLm.nearbyLandmarks.some(origLm => lm == origLm.id))

        store.nearbyLm.setNearbyLandmarks(newLandmarksNearUser)

        // TODO: redo notifications
        // // if there are any new landmarks near user, create a notification for them and send it
        // if (newLandmarksNotPreviouslyNearUser?.length > 0) {
        //     const body = newLandmarksNotPreviouslyNearUser.length > 1 ? "There are new landmarks near by. Tap here to view" : "There is a new landmark close by. Tap here to view"
        //     const notifType: NotifType = newLandmarksNotPreviouslyNearUser.length > 1 ? 'near-landmarks' : 'near-landmark'
        //     const data = { notif_type: notifType, landmarks: newLandmarksNotPreviouslyNearUser.length == 1 ? newLandmarksNearUser : null }
        //     await Notifications.scheduleNotificationAsync({
        //         content: {
        //             title: "⚠ Landmarks close by ⚠",
        //             body: body,
        //             data: data
        //         },
        //         trigger: { seconds: 2 },
        //     });
        // }
    }

    const addLandmark = async () => {
        await store.addLm.stageNewLandmarkWithLocation({longitude: userLocation.longitude, latitude: userLocation.latitude}, -1)
    }
    
    return {flyToUser, mapRef, loading, getInitialRegion, onLocationUpdate, followUser, goIndoors, landmarks, userLocation, addLandmark}
}