﻿/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import { FontAwesome } from "@expo/vector-icons";
import { RouteProp, useNavigation } from "@react-navigation/native";
import { observer } from "mobx-react";
import React, { useState } from "react";
import { ActivityIndicator, Alert, Image, Keyboard, Linking, Modal, StyleSheet, Text, TouchableOpacity, TouchableWithoutFeedback, View } from "react-native";
import MapView, { MapPolygonProps, MapPolylineProps, Marker, Polygon, Polyline } from "react-native-maps";
import { Menu, MenuProps } from "react-native-material-menu";
import { useLandmarks } from "../../../api/landmarks";
import { store } from "../../../main-store";
import { usePermissions } from "../../../permissions-context";
import { colors, lmTypes } from "../../../utils/GlobalUtils";
import Badge from "../../Badge";
import { BurgerMenu } from "../../burger-menu";
import { IconButton } from "../../Buttons";
import { MainTabsParamList } from "../../navigation/main-tabs-navigator";
import { MapStackParamList } from "../map-navigator";
import mapStyles from "../Map.styles";
import NearbyLandmarksPanel from "../panels/nearby-landmarks-panel/neary-landmarks-panel";
import { VoicePanel } from "../panels/voice-panel/voice-panel.view";
import { useOutdoorMap } from "./outdoor-map.logic";

/**
 * An interface representing the user location retrieved from [expo-location]{@link https://docs.expo.dev/versions/latest/sdk/location/}.
 * @category Map
 */
export interface UserLocation {
    latitude: number;
    longitude: number;
    heading?: number;
}

export type MapStackRouteProp = RouteProp<MapStackParamList, 'Outdoor'>;
export type AuthTabsMapRouteProp = RouteProp<MainTabsParamList, 'Map'>;

/**
 * The screen component containing the Map and all related functionality. Uses [react-native-maps]{@link https://github.com/react-native-maps/react-native-maps}
 * @category Map
 * @component
 */

// TODO: replace this with buildings api
const cameronPolyProps: MapPolygonProps = {
    coordinates: [
        { latitude: 53.527190, longitude: -113.524205 },
        { latitude: 53.526510, longitude: -113.524205 },
        { latitude: 53.526510, longitude: -113.523452 },
        { latitude: 53.527190, longitude: -113.523452 },
    ],
    fillColor: `rgba(100,100,200,0.3)`,
    strokeWidth: 2.5,
    tappable: true,
};

const busStationToCameronRouteProps: MapPolylineProps = {
    coordinates: [
        {latitude: 53.527192, longitude: -113.523583},
        {latitude: 53.527189, longitude: -113.5233285},
        {latitude: 53.526942, longitude: -113.523338},
        {latitude: 53.526934, longitude: -113.523263},
        {latitude: 53.5264874, longitude: -113.5232654},
        {latitude: 53.526492, longitude: -113.522827},
        {latitude: 53.52620959999999,longitude: -113.5228282},
        {latitude: 53.5259882, longitude: -113.5222835},
        {latitude: 53.5254566, longitude: -113.5222824},
        {latitude: 53.525456, longitude: -113.522155},
        {latitude: 53.5251288, longitude: -113.5216083}
    ],
    strokeColor: 'black',
    strokeWidth: 3,
    onPress:() => Alert.alert("This is a route from University Station to Cameron Library"),
    tappable: true
}

const coordinates = [
        { latitude: 53.527086340019856, longitude: -113.52358410971608, }, // Cameron library
        { latitude: 53.52516024715472, longitude: -113.52154139033108, }, // University station
    ];

const RefreshPanel: React.FC<{loading: boolean}> = ({loading}) => {
    return (
        <Modal transparent={true} animationType="fade" visible={loading}>
            <View style={{ flex: 1, justifyContent: 'center', alignItems: 'center', backgroundColor: 'rgba(0,0,0,0.3)' }}>
                <View style={{ width: '60%', height: '30%', backgroundColor: colors.red, justifyContent: 'center', alignItems: 'center', borderRadius: 20 }}>
                    <ActivityIndicator size="large" color="white" style={{ marginBottom: 20 }} />
                    <Text style={{ fontSize: 15, color: 'white' }}>Refreshing</Text>
                </View>
            </View>
        </Modal>
    )
}

const LandmarkPins: React.FC = () => {
    const {data: landmarks} = useLandmarks();

    return (
        <>
        {landmarks?.map((landmark) => {
            if (landmark.floor == null) {
                return (
                    <Marker
                        tracksViewChanges={landmark?.id == store.selectedLm.selectedLandmarkId}
                        onPress={() => store.selectedLm.selectLandmark(landmark.id)}
                        key={landmark.id}
                        coordinate={{ latitude: landmark.latitude as number, longitude: landmark.longitude as number }} >
                        {landmark.landmark_type ? <Image style={{ height: 35, width: 25 }} source={lmTypes[landmark.landmark_type]?.image} /> : null}
                    </Marker>
                )
            }
        })}
        </>
    )
}

const OutdoorMap: React.FC = (props) => {
    const {
        loading, 
        mapRef, 
        followUser, 
        addLandmark,
        flyToUser,
        goIndoors, 
        getInitialRegion, 
        onLocationUpdate,
    } = useOutdoorMap();
        
    const {locationPermissionsGranted, voicePermissionsGranted} = usePermissions();
    const navigation = useNavigation();
    const [mapSize, setMapSize] = useState<number>();

    const map = () => {
        return (
            <MapView
                toolbarEnabled={false}
                onPress={() => Keyboard.dismiss()}
                testID="mapView"
                ref={mapRef}
                style={{ width: '100%', height: '100%' }}
                initialRegion={getInitialRegion()}
                onLongPress={e => store.addLm.stageNewLandmarkWithLocation(e.nativeEvent.coordinate)}
                showsUserLocation={locationPermissionsGranted}
                onUserLocationChange={e => onLocationUpdate(e.nativeEvent.coordinate)}
                followsUserLocation={followUser}
                showsMyLocationButton={false}
                onRegionChangeComplete={(region) => setMapSize(region.latitudeDelta)}>
                <Polygon {...cameronPolyProps} onPress={goIndoors}/>
                <LandmarkPins/>
                <Polyline {...busStationToCameronRouteProps}/>
                <Marker coordinate={coordinates[1]} pinColor={colors.red}/>
                <Marker coordinate={{  latitude: 53.527189,longitude: -113.5233285, }} pinColor={colors.red}>
                    <Text style={{ fontSize: mapSize > 0.00327 ? 0 : 0.05 / mapSize, maxWidth: 200 }}>Route from University Station to Cameron Library</Text>
                </Marker>
            </MapView>
        )
    }
        
    return (
        <TouchableWithoutFeedback>
            <>
                {/*Main map component*/}
                <RefreshPanel loading={loading}/>
                {map()}
                <BurgerMenu 
                    styles={{top: 60, right: 20, position: 'absolute'}}
                    menuItems={[
                        {onPress: () => {}, text: 'Indoor buildings', disabled: true, extraProps: {disabledTextColor: 'black', style: styles.burgerMenuHeaderItem}},
                        {onPress: () => navigation.navigate("Indoor" as any), text: 'Cameron'},
                        {onPress: () => {}, text: 'More maps on the way!', disabled: true}
                    ]}/>
                {/*Map buttons*/}

                {/*Near landmark button*/}
                {store.nearbyLm.nearbyLandmarks?.length > 0 &&
                <IconButton size={20} color="white" style={[mapStyles.lowerMapButton, mapStyles.alertButton]} icon="exclamation-triangle" onPress={() => store.nearbyLm.focusNearbyLandmarks()}>
                    <Badge positioning={{ bottom: 7, right: 4 }} value={store.nearbyLm.nearbyLandmarks.length} />
                </IconButton>}
                {/*Voice button*/}
                {locationPermissionsGranted && voicePermissionsGranted &&
                <IconButton size={20} color='white' style={[mapStyles.lowerMapButton, mapStyles.voiceButton]} icon="microphone" onPress={async () => await store.voice.startSpeech()} />}
                {/*Add button*/}
                <IconButton size={20} color='white' style={[mapStyles.lowerMapButton, mapStyles.addLandmarkButton]} icon="plus" onPress={addLandmark} />
                {/*Center map on user button*/}
                <IconButton size={20} color='white' style={[mapStyles.lowerMapButton, mapStyles.userLocationButton]} icon="location-arrow" onPress={flyToUser} />

                {/*Map Panels*/}
                <NearbyLandmarksPanel/>
                {locationPermissionsGranted && voicePermissionsGranted && <VoicePanel/>}
            </>
        </TouchableWithoutFeedback>)
}

const styles = StyleSheet.create({
    burgerMenuHeaderItem: { alignItems: 'center', borderColor: 'black', borderBottomWidth: 1, marginHorizontal: 10, opacity: .7 }
})

export default observer(OutdoorMap);