﻿/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import FontAwesome from "@expo/vector-icons/build/FontAwesome";
import { observer } from "mobx-react-lite";
import React, { memo, useEffect, useRef, useState } from "react";
import { ActivityIndicator, Dimensions, Image, ImageSourcePropType, Keyboard, KeyboardAvoidingView, KeyboardEventName, Platform, SafeAreaView, StyleSheet, Text, TextInput, TouchableOpacity, View } from 'react-native';
import { ScrollView } from "react-native-gesture-handler";
import Modal from 'react-native-modal';
import { Image as ImageSVG, Svg } from 'react-native-svg';
import ViewShot from "react-native-view-shot";
import { store } from "../../../../main-store";
import { colors, lmTypes } from "../../../../utils/GlobalUtils";
import { IconButton, SecondaryButton } from "../../../Buttons";
import LandmarkTypePicker from "../../../LandmarkTypePicker";
import { PhotoPicker } from "../../../PhotoPicker";
import { Separator } from "../../../Separator";
import IndoorFloor from "../../indoor/indoor-floor";
import TouchOpaq from "../selected-landmark-panel/TouchOpaq";
import { useAddLandmarkApi } from "./add-landmark-panel.api";

const imgWidth = 346;
const imgHeight = 448;
const imageDim = 25;

/**
 * Component that renders a form for adding a new {@link Landmark}. Contained within a [react-native-modal]{@link https://github.com/react-native-modal/react-native-modal}.
 * @component
 * @category Map
 */
const AddLandmarkPanel: React.FC = observer(() => {
    const {lmLocationScreenCap, addLandmarkMutation, submit} = useAddLandmarkApi();
    const [keyboardOpened, setKeyboardOpened] = useState<boolean>(false);
    
    useEffect(() => {
        let eventString = Platform.OS == "android" ? 'keyboardDidShow' : Platform.OS == "ios" ? 'keyboardWillShow' : null;
        if (eventString) {
            const validEventString: KeyboardEventName = eventString as KeyboardEventName;

            const keyboardDidShowListener = Keyboard.addListener(
                validEventString,
                () => {
                    console.log('keyboard open')
                    setKeyboardOpened(true); // or some other action
                }
            );
            const keyboardDidHideListener = Keyboard.addListener(
                'keyboardDidHide',
                () => {
                    console.log('keyboard close')
                    setKeyboardOpened(false); // or some other action
                }
            );

            return () => {
                keyboardDidHideListener.remove();
                keyboardDidShowListener.remove();
            };
        }
    }, []);

    /**
     * Returns a height for the modal depending on if an image is maximzed, if the keyboard is opened, and if the current landmark has photos associated with it
     */
     const determineModalHeight = () => {
        if (keyboardOpened) {
            return Dimensions.get("window").height * .45
        }
        else if (store.addLm.photos?.length > 0) {
            return Dimensions.get("window").height * .9
        }
        else {
            return Dimensions.get("window").height * .8
        }
    }

    return (
        <Modal
            useNativeDriver={true}
            useNativeDriverForBackdrop={true}
            testID="addLMModal"
            avoidKeyboard={store.addLm.photos?.length > 0}
            onBackdropPress={store.addLm.close}
            style={{ flex: 1, justifyContent: "flex-end", height: '100%', margin: 0}}
            isVisible={store.addLm.panelVisible} >

            <KeyboardAvoidingView
                behavior={Platform.OS === "ios" ? "padding" : "height"}
                enabled={store.addLm.photos?.length > 0}
                style={{ flex: 1, justifyContent: "flex-end"}}
            >
                <SafeAreaView style={{ backgroundColor: colors.red, height: determineModalHeight(), justifyContent: 'flex-end'}}>
                    {addLandmarkMutation.isIdle ?
                        <>
                            <View style={{
                                justifyContent: 'space-between',
                                alignItems: 'center',
                                flexDirection: "row",
                                marginBottom: 15,
                                borderBottomWidth: 1,
                                borderBottomColor: 'white',
                                paddingHorizontal: 20,
                                paddingVertical: 10,
                            }}
                            onTouchStart={() => Keyboard.dismiss()}
                            >
                                <Text style={{ color: 'white', fontSize: 15 }}>Add landmark here?</Text>
                                <TouchOpaq
                                    func={store.addLm.close}
                                    name={"close"}
                                    size={25}
                                    col={"white"}
                                />

                            </View>
                            <ScrollView>
                                <View style={{ paddingHorizontal: 20, paddingBottom: 20 }}>
                                    <TextInput
                                        returnKeyType="done"
                                        blurOnSubmit={true}
                                        multiline={true}
                                        style={{ backgroundColor: 'white', textAlignVertical: 'top', paddingHorizontal: 10, paddingTop: 10, paddingBottom: 10, marginBottom: 20, height: 150 }}
                                        placeholder="Description"
                                        onChangeText={value => store.addLm.updatePendingLandmark({ ...store.addLm.pendingLandmark, description: value })}>
                                        {store.addLm.pendingLandmark?.description}
                                    </TextInput>
                                    <View style={{ flexDirection: 'row' }}>
                                        <LandmarkTypePicker 
                                            placeholder={{ label: "Select a landmark type...", value: 0 }}
                                            value={store.addLm.pendingLandmark?.landmark_type}
                                            onValueChange={(value) => {
                                                if (value) {
                                                    store.addLm.updatePendingLandmark({ ...store.addLm.pendingLandmark, landmark_type: value, title: lmTypes[value].label })
                                                }
                                                else {
                                                    store.addLm.updatePendingLandmark({ ...store.addLm.pendingLandmark, landmark_type: undefined, title: 'no title' })
                                                }
                                            }}
                                            items={Object.keys(lmTypes)?.map(icon => {
                                                return (
                                                    { label: lmTypes[parseInt(icon)]?.label.toUpperCase(), value: icon, key: icon }
                                                )
                                            })}/>
                                        {store.addLm.pendingLandmark?.landmark_type ? <Image source={lmTypes[store.addLm.pendingLandmark.landmark_type].image} />
                                            : null}
                                    </View>
                                </View>
                                <View style={{ justifyContent: 'flex-end', flexDirection: 'row', paddingHorizontal: 20, marginTop: 5 }}> 
                                    <View style={{ flexDirection: 'row' }}>
                                        <TouchableOpacity onPress={async () => await submit()}><Text style={{ color: 'white', marginRight: 25, opacity: store.addLm.landmarkReady ? 1 : .5 }}>Add</Text></TouchableOpacity>
                                        <TouchableOpacity onPress={store.addLm.close}><Text style={{ color: 'white', marginRight: 25 }}>Cancel</Text></TouchableOpacity>
                                    </View>
                                </View>
                                {store.addLm.photos?.length > 0 && !keyboardOpened ?
                                    <>
                                        <ScrollView style={{ borderTopWidth: 1, borderColor: 'lightgray', paddingTop: 20, marginHorizontal: 20, flexDirection: 'row', marginBottom: 5, marginTop: 30 }} horizontal={true}>
                                            {store.addLm.photos.map((photo, i) => {
                                                return (
                                                    <View key={i} style={{ marginHorizontal: 1, padding: 15 }}>
                                                        <IconButton style={{ position: 'absolute', top: 0, right: 0, zIndex: 10, }} icon="times-circle" color="lightgray" size={20} onPress={() => store.addLm.deletePhoto(i)} />
                                                        <Image style={{ borderWidth: 1, alignSelf: 'center', height: 200, width: 200 * photo.width / photo.height }} source={{ uri: photo.image_b64 }} />
                                                    </View>
                                                )
                                            })}
                                            {store.addLm.photos.length < 5 ? <IconButton style={{ alignSelf: 'center', padding: 10, opacity: .5, marginLeft: 10 }} color='white' size={30} icon="plus" onPress={() => store.addLm.togglePhotoSourceMenu(true)} /> : null}
                                        </ScrollView>
                                    </> : null}
                                    {store.addLm.photos?.length == 0 && !keyboardOpened ?
                                    <>
                                        <Separator color="white" style={{marginHorizontal: 20, marginTop: 20}} />
                                        <TouchableOpacity style={{height: '30%', alignItems: 'center', justifyContent: 'center', marginBottom: 20}} onPress={() => {store.addLm.togglePhotoSourceMenu(true)}}>
                                            <Text style={{fontSize: 20, marginBottom: 10, color: 'white'}}>Add photo of landmark</Text>
                                            <FontAwesome name="plus" size={30} color='white' />
                                        </TouchableOpacity> 
                                    </>: null}
                            </ScrollView>
                        </> :
                        <View style={{ height: '100%', justifyContent: "space-evenly", alignItems: "center" }}>
                            <Text style={{ color: 'white', fontSize: 20 }}>{
                                addLandmarkMutation.isLoading ? 'Uploading landmark...' :
                                addLandmarkMutation.isError ? 'Something went wrong when trying to upload the landmark.' : null}
                            </Text>
                            {
                                addLandmarkMutation.isLoading ? <ActivityIndicator color='white' size="large" /> :
                                addLandmarkMutation.isError ? <SecondaryButton text="Okay" onPress={store.addLm.close} /> : null
                            }
                        </View>}
                </SafeAreaView>
                <PhotoPicker multiple={true} menuType='alert' photoSourceMenuOpened={store.addLm.photoSourceMenuVisible} onReceivedPhotoResult={result => store.addLm.addPhoto(result)} cancel={() => store.addLm.togglePhotoSourceMenu(false)} />

                <ViewShot style={{ width: imgWidth + 20, height: imgHeight + 20, position: 'absolute', right: -2000 }} ref={lmLocationScreenCap} >
                    {/* {console.log("newLandmark is " + newLandmark)} */}
                    {store.addLm.pendingLandmark == null || store.addLm.pendingLandmark.floor == null || store.addLm.pendingLandmark.landmark_type == null ? <></> :
                        <View style={styles.container}>
                            <Svg>
                                <IndoorFloor floorNum={store.addLm.pendingLandmark.floor} />
                                <ImageSVG x={store.addLm.pendingLandmark.longitude * imgWidth - 3} y={store.addLm.pendingLandmark.latitude * imgHeight - 3} width={imageDim} height={imageDim} href={lmTypes[store.addLm.pendingLandmark.landmark_type]['image'] as ImageSourcePropType} />
                            </Svg>
                        </View>
                    }
                </ViewShot>
            </KeyboardAvoidingView>
        </Modal>

    )
})

const styles = StyleSheet.create({
    container: {
        aspectRatio: 8 / 10,  // (caters to portrait mode) (width is 80% the value of height dimension)
        width: '100%',
        height: '100%',
        maxWidth: "100%",
        maxHeight: "100%",
        backgroundColor: "white",
        padding: 10
    }
})



export default memo(AddLandmarkPanel);
