﻿/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import { FontAwesome } from "@expo/vector-icons";
import { observer } from "mobx-react";
import React from "react";
import { ActivityIndicator, Dimensions, Image, Keyboard, SafeAreaView, ScrollView, StyleSheet, Text, TouchableOpacity, View } from "react-native";
import Modal from 'react-native-modal';
import { useLandmark } from "../../../../api/landmarks";
import { store } from "../../../../main-store";
import { colors } from "../../../../utils/GlobalUtils";
import { IconButton, PrimaryButton } from "../../../Buttons";
import { DetailsBody } from "./DetailsBody";
import { DetailsHeader } from "./DetailsHeader";
import { useSelectedLandmarkApi } from './selected-landmark-panel.api';

/**
 * Component that renders the details of a selected {@link Landmark} and allows the user to edit those details. Contained within a [react-native-modal]{@link https://github.com/react-native-modal/react-native-modal}.
 * @component
 * @category Map
 */
export const LandmarkDetails: React.FC = observer(({}) => {
    const {determineModalHeight, tryDeleteSelectedPhoto, mainScrollRef} = useSelectedLandmarkApi();
    const landmarkQuery = useLandmark(store.selectedLm.selectedLandmarkId);

    return (
        <Modal 
            useNativeDriver={true}
            useNativeDriverForBackdrop={true}
            avoidKeyboard={true}
            onBackdropPress={() => {
                if (store.selectedLm.editing) {
                    Keyboard.dismiss();
                } else {
                    store.selectedLm.close();
                }
            }}
            style={{justifyContent: "flex-end", height: '100%', margin: 0}}
            isVisible={store.selectedLm.panelVisible}>
            <SafeAreaView 
                style={[styles.container, {height: determineModalHeight()}]}>

                {store.selectedLm.selectedImageIndex > -1 ?
                <View style={{ padding: 14}}>
                    <View style={{justifyContent: 'space-between', flexDirection: 'row'}}>
                        <TouchableOpacity onPress={() => store.selectedLm.selectImage(-1)}>
                            <>
                                <Text style={{color: 'white', fontSize:20}}><FontAwesome name="arrow-left" color="white" size={20}/> Back</Text>
                            </>
                        </TouchableOpacity>
                        <IconButton 
                            style={{alignSelf: 'flex-end', marginBottom: 20}} 
                            icon="trash" 
                            color="white" 
                            size={25} 
                            onPress={() => tryDeleteSelectedPhoto()}/>
                    </View>
                    <ScrollView style={{width: '100%'}} ref={mainScrollRef}>
                        <Image style={{resizeMode: 'contain', alignSelf: 'center', height: Dimensions.get('window').height * .9, width: Dimensions.get('window').width}} source={{uri: 'data:image/png;base64,' + landmarkQuery?.data?.landmark?.photos[selectedImage].image_b64}}/> 
                    </ScrollView>
                </View>
                :
                (landmarkQuery.isSuccess || landmarkQuery.isIdle) && 
                (editLandmarkMutation.isIdle || editLandmarkMutation.isSuccess) && 
                (deleteLandmarkMutation.isIdle || deleteLandmarkMutation.isSuccess) ?
                <>
                <DetailsHeader
                    authNavigation={authNavigation}
                    toggleLmDetails={toggleLmDetails}
                    ratedByUser={landmarkQuery?.data?.ratedByUser}
                    processingPhoto={processingPhoto}
                    addPhotoStatus={addLandmarkPhotoMutation.status}
                    deletePhotoStatus={deleteLandmarkPhotoMutation.status}
                    toggleDetailsPanel={toggleDetailsPanel}
                    landmark={landmarkQuery?.data?.landmark}
                    editLandmark={editLandmark}
                    editingEnabled={editingEnabled}
                    toggleEditing={setEditing}
                    rateLandmark={rateLandmark}
                    removeLandmark={removeLandmark}
                    updatedLandmark={updatedLandmark}
                    profile={profile} />
                <DetailsBody
                    authNavigation={authNavigation} 
                    setProcessingPhoto={setProcessingPhoto}
                    processingPhoto={processingPhoto}
                    profileId={profile?.id}
                    setKeyboardOpened={setKeyboardOpened}
                    keyboardOpened={keyboardOpened}
                    setSelectedImage={setSelectedImage}
                    landmark={landmarkQuery?.data?.landmark}
                    updatedLandmark={updatedLandmark}
                    commentListRef={commentListRef}
                    commentTextInputRef={commentTextInputRef}
                    commentBeingEdited={commentBeingEdited}
                    comments={commentsQuery?.data}
                    addComment={addComment}
                    setCommentBeingEdited={setCommentBeingEdited}
                    newCommentId={newCommentId}
                    editComment={editComment}
                    focusComment={focusComment}
                    deleteComment={deleteComment}
                    setNewComment={setNewComment}
                    focusedCommentId={focusedCommentId}
                    startEditingComment={startEditingComment}
                    editingEnabled={editingEnabled}
                    setUpdatedLandmark={setUpdatedLandmark}
                    tryDeletePhoto={tryDeleteSelectedPhoto}
                    deletePhotoStatus={deleteLandmarkPhotoMutation.status}
                    toggleLmDetails={toggleLmDetails}
                    addPhoto={addLandmarkPhotoMutation.mutateAsync}
                    addPhotoStatus={addLandmarkPhotoMutation.status}/>
                </> :
                <View style={{height: '100%', justifyContent: "space-evenly", alignItems: "center", marginHorizontal: 20}}>
                    <Text style={{color: 'white', fontSize: 20}}>{
                        landmarkQuery.isLoading ? "Loading landmark..." : 
                        landmarkQuery.isError ? "Something went wrong trying to load the landmark" :
                        deleteLandmarkMutation.isLoading ? "Deleting landmark..." : 
                        deleteLandmarkMutation.isError ? "Something went wrong trying to delete the landmark" :
                        editLandmarkMutation.isLoading ? "Updating landmark..." : 
                        editLandmarkMutation.isError ? "Something went wrong trying to update the landmark" :
                        addCommentMutation.isLoading ? "Adding comment..." : 
                        addCommentMutation.isError ? "Something went wrong trying to add the comment" : 
                        editCommentMutation.isLoading ? "Updating comment..." : 
                        editCommentMutation.isError ? "Something went wrong trying to update the comment" :  
                        deleteCommentMutation.isLoading ? "Deleting comment..." : 
                        deleteCommentMutation.error ? "Something went wrong trying to delete the comment" : null} 
                    </Text>
                    {
                        deleteLandmarkMutation.isLoading || editLandmarkMutation.isLoading || landmarkQuery.isLoading ? <ActivityIndicator color='white' size="large"/> :
                        deleteLandmarkMutation.isError || editCommentMutation.isLoading || landmarkQuery.isError ? <PrimaryButton text="Okay" style={{borderColor: 'white', borderWidth: 1}} onPress={async () => await landmarkQuery.refetch()} /> : null
                    }
                </View>}
            </SafeAreaView>
        </Modal>
    )
})

const styles = StyleSheet.create({
    container: {
        backgroundColor: colors.red,
    },
    detailsContainer: {
        flex: 1,
        marginHorizontal: 20
    },

    commentContainer: {
        flex: 5,
        marginBottom: 40,
    },
    title: {

    },
    input: {
        padding: 5,
        color: 'black'
    }
})