import { useEffect, useRef, useState } from "react";
import { Alert, Dimensions, FlatList, Keyboard, ScrollView, TextInput } from "react-native";
import { useAddComment, useDeleteComment, useEditComment, useLandmarkComments } from "../../../../api/comments";
import { useDeleteLandmark, useEditLandmark, useLandmark, useRateLandmark } from "../../../../api/landmarks";
import { useDeleteLandmarkPhoto } from "../../../../api/lm-photos.ts";
import { useOwnedProfile } from "../../../../api/profile/profiles.get-own";
import { store } from "../../../../main-store";
import { useAuth } from "../../../../state/external/auth-provider";
import { Landmark, LMComment } from "../../../../types";

export const useSelectedLandmarkApi = () => {
    const landmarkQuery = useLandmark(store.selectedLm.selectedLandmarkId)
    const editLandmarkMutation = useEditLandmark();
    const rateLandmarkMutation = useRateLandmark();
    const deleteLandmarkMutation = useDeleteLandmark();
    const commentsQuery = useLandmarkComments(store.selectedLm.selectedLandmarkId);
    const addCommentMutation = useAddComment();
    const editCommentMutation = useEditComment();
    const deleteCommentMutation = useDeleteComment();
    const deleteLandmarkPhotoMutation = useDeleteLandmarkPhoto();
    
    const {userId, landmarkOwnedByUser} = useAuth()

    // /**
    //  * Holds the state of the {@link Landmark} being displayed.
    //  */
    // const selectedLandmarkState = undefined;
    // const [selectedLandmark, setLandmark] = useState<Landmark | undefined>(selectedLandmarkState);
    /**
     * Holds state of a {@link Landmark} object parallel to {@linkcode selectedLandmarkState} that is manipulated when {@linkcode editingEnabled} is true.
     */
    const [updatedLandmark, setUpdatedLandmark] = useState<Landmark | undefined>(undefined);
    /**
     * Holds text of the current {@link LMComment} being added to this {@link Landmark}.
     */
     const [newCommentId, setNewComment] = useState<string>("");
    /**
     * Holds id of the focused {@link LMComment} in the comment list.
     */
    const [focusedCommentId, setFocusedComment] = useState<string>("");
    /**
     * Holds state of the {@link LMComment} being currently edited.
     */
     const [commentBeingEdited, setCommentBeingEdited] = useState<LMComment | undefined>(undefined);

    /**
     * Flag that tracks if the keyboard is open
     */
    const [keyboardOpened, setKeyboardOpened] = useState<boolean>(false);  

    const [photosBusy, setPhotosBusy] = useState<boolean>(false)
    const [processingPhoto, setProcessingPhoto] = useState<boolean>(false)    
    const { profile } = useOwnedProfile()

    /**
     * Holds a reference to the Flatlist containing the comments.
     */
    const commentListRef = useRef<FlatList>();
    const mainScrollRef = useRef<ScrollView>();
    /**
     * Holds a reference to the text input for posting a new comment
     */
    const commentTextInputRef = useRef<TextInput>();

    useEffect(() => {
    const keyboardDidShowListener = Keyboard.addListener(
      'keyboardWillShow',
      () => {
        setKeyboardOpened(true); // or some other action
      }
    );
    const keyboardDidHideListener = Keyboard.addListener(
      'keyboardWillHide',
      () => {
        setKeyboardOpened(false); // or some other action
      }
    );

    return () => {
      keyboardDidHideListener.remove();
      keyboardDidShowListener.remove();
    };
  }, []);

    useEffect(() => {
        /**
         * Resets the {@linkcode rateLandmark} mutation on successful update.
         * Embedded in a useEffect that listens to the {@linkcode rateLandmarkStatus} value from the {@link useLandmarks} hook.
         * @memberOf LandmarkDetails
         */
         const resetUpdateLandmark = async () => {
            if (rateLandmarkMutation.isSuccess) {
                rateLandmarkMutation.reset()
                await landmarkQuery.refetch()
            }

            if (editLandmarkMutation.isSuccess) {
                editLandmarkMutation.reset()
                await landmarkQuery.refetch()
            }
        }
        resetUpdateLandmark();
    }, [rateLandmarkMutation.status, editLandmarkMutation.status]);

    useEffect(() => {
        /**
         * Resets the {@linkcode updateComment} mutation on successful update.
         * Embedded in a useEffect that listens to the {@linkcode updateCommentStatus} value from the {@link useComments} hook.
         * @memberOf LandmarkDetails
         */
         const resetUpdateComment = () => {
            if (editCommentMutation.isSuccess) {
                commentListRef.current?.scrollToItem({animated: true, item: commentBeingEdited});
                setCommentBeingEdited(undefined)
                Keyboard.dismiss();
            }
        }
        resetUpdateComment();
    }, [editCommentMutation.status]);

    useEffect(() => {
        /**
         * Resets the {@linkcode deleteLandmark} mutation on successful delete.
         * Embedded in a useEffect that listens to the {@linkcode deleteLandmarkStatus} value from the {@link useLandmarks} hook.
         * @memberOf LandmarkDetails
         */
        const resetDeleteLMOnSuccess = () => {
            if (deleteLandmarkMutation.isSuccess) {
                deleteLandmarkMutation.reset()
            }

            console.log(deleteLandmarkMutation.status)
        }
        resetDeleteLMOnSuccess();
    }, [deleteLandmarkMutation.status]);

    useEffect(() => {
        /**
         * Resets the {@link addCommentAsync} mutation on successful add.
         * Embedded in a useEffect that listens to the {@link addCommentAsync} value from the {@link useComment} hook.
         * @memberOf LandmarkDetailsLandmark
         */
        const resetAddCommentOnSuccess = () => {
            if (addCommentMutation.isSuccess) {
                addCommentMutation.reset()
            }
            setNewComment('')
            commentListRef.current?.scrollToIndex({animated: true, index: 0});
            Keyboard.dismiss();
        }
        resetAddCommentOnSuccess();
    }, [addCommentMutation.status]);

    useEffect(() => {
        /**
         * Clears selected comment when {@linkcode newCommentId} state changes.
         * @memberOf LandmarkDetails
         */
        const clearSelectedOnNewCommentChange = () => {
            setFocusedComment('')
        }
        clearSelectedOnNewCommentChange();
    }, [newCommentId]);

    /**
     * Calls the {@linkcode updateLandmark} mutation from the {@link useLandmarks} hook and closes the modal once finished.
     */
    const editLandmark = async () => {
        if (updatedLandmark) {
            await editLandmarkMutation.mutateAsync(updatedLandmark) 
        }
        
        store.selectedLm.setEditing(false);
    }

    /**
     * Calls the {@linkcode rateLandmarkAsunc} mutation from the {@link useLandmarks} hook. If 1, the landmark will be upvoted. If -1, it will be downvoted
     */
     const rateLandmark = async (rating: 1 | -1) => {
        if (landmarkQuery?.data?.landmark) {
            await rateLandmarkMutation.mutateAsync({id: store.selectedLm.selectedLandmarkId, rating: rating});
        }
    }

    /**
     * Calls the {@linkcode deleteLandmark} mutation from the {@link useLandmarks} hook and closes the modal once finished.
     */
    const removeLandmark = async () => {
        Alert.alert("Are you sure you want to delete this landmark?", undefined,
      [{ text: "Cancel", }
        ,
      {
        text: "Confirm", onPress: async () => {
            await deleteLandmarkMutation.mutateAsync(store.selectedLm.selectedLandmarkId);   
            store.selectedLm.close();
            Alert.alert("Landmark Deleted", "This landmark has been deleted.");
        }
      }])
    }

    /**
     * Calls the {@linkcode addComment} mutation from the {@link useComments} hook.
     */
    const addComment = async () => {
        if (newCommentId) {
            await addCommentMutation.mutateAsync({
                edited: false,
                content: newCommentId,
                landmark: store.selectedLm.selectedLandmarkId,
                poster: userId,
                poster_name: profile.username,
                id: ''
            });
        }
    }

    /**
     * Set a comment to be focused in the comment box. It will expand and show delete and edit controls for that comment. 
     * If the comment id given is already selected, the comment will be unselected
     */
    const focusComment = (id: string) => {
        if (focusedCommentId == id) {
            setFocusedComment("");
        }
        else {
            setFocusedComment(id);
        }
    }

    /**
     * Set a comment to be the comment currently edited. 
     */
    const startEditingComment = (comment: LMComment) => {
        setCommentBeingEdited(comment);
        commentTextInputRef.current.focus();
    }

    /**
     * Calls the {@linkcode editComment} mutation from the {@link useComments} hook.
     */
    const editComment = async (comment: LMComment) => {
        await editCommentMutation.mutateAsync(comment);
    }

    /**
     * Calls the {@linkcode deleteComment} mutation from the {@link useComments} hook.
     */
    const deleteComment = async (id: string) => {
        await deleteCommentMutation.mutateAsync(id);
    }

    const getWindowWidth = () => {
        return Dimensions.get('window').width
    }

    /**
     * Prompts user for a confirmation to delete the photo that they just tried to delete
     */
    const tryDeleteSelectedPhoto = () => {
        const photoId = landmarkQuery?.data?.landmark?.photos[store.selectedLm.selectedImageIndex].id
        Alert.alert(
            'Confirm photo removal', 
            'Are you sure you want to delete this photo?', 
            [
                {text: 'Yes', 
                onPress: async () => {
                    setPhotosBusy(true)
                    await deleteLandmarkPhotoMutation.mutateAsync(photoId)
                    store.selectedLm.selectImage(-1)
                    await landmarkQuery.refetch()
                }},
                {text: 'No'} 
            ]
        )  
    }

    /**
     * Returns a height for the modal depending on if an image is maximzed, if the keyboard is opened, and if the current landmark has photos associated with it
     */
    const determineModalHeight = () => {
        if (store.selectedLm.selectedImageIndex > -1) 
            return Dimensions.get("window").height 
        else if (keyboardOpened || store.selectedLm.editing || (!landmarkOwnedByUser(landmarkQuery?.data?.landmark) && landmarkQuery?.data?.landmark?.photos?.length == 0)) {
            return Dimensions.get("window").height * .4
        }
        else if (landmarkQuery?.data?.landmark?.photos?.length > 0) 
            return Dimensions.get("window").height * .9 
        else
            return Dimensions.get("window").height * .6
    }

    return {
        determineModalHeight, tryDeleteSelectedPhoto, mainScrollRef,
        landmarkQuery, editLandmarkMutation, deleteLandmarkMutation,
        addCommentMutation, editCommentMutation, deleteCommentMutation,
    };
    // (landmarkQuery.isSuccess || landmarkQuery.isIdle) && 
    //             (editLandmarkMutation.isIdle || editLandmarkMutation.isSuccess) && 
    //             (deleteLandmarkMutation.isIdle || deleteLandmarkMutation.isSuccess)
}