import { makeAutoObservable } from "mobx"
import Spokestack, { PipelineProfile } from 'react-native-spokestack'
import { MainStore } from "../../../../main-store";
import { lmTypes } from "../../../../utils/GlobalUtils";
import { AddLandmarkStore } from "../add-landmark-panel/add-landmark-panel.store";

const STOP_RESPONSES = ["cancel", "stop", "goodbye", "bye"]

type ActionVerb = "add" | "view" | "near";
type ActionVerbMap = Record<ActionVerb, string[]>
const ACTION_VERB_MAP: ActionVerbMap = {
    'add': ["add", "create", "make"],
    'view': ["view", "look", 'show'],
    'near': ["near", "close"]
}

type ActionStepHandlerMap = Record<number, () => void>;
type ActionHandlerMap = Record<ActionVerb, ActionStepHandlerMap>;

export interface VoiceAction {
    verb: ActionVerb
    step: number
}

export class VoicePanelStore {
    panelVisible = false;
    listening = false;
    partialSpeechResults?: string;
    lastSaid?: string;
    previousResult?: string;
    action?: VoiceAction;
    feedback?: string;
    addLmStore: AddLandmarkStore;

    private actionHandlerMap: ActionHandlerMap = {
        'add': {
            1: async () => {
                this.addLmStore.updatePendingLandmark({...this.addLmStore.pendingLandmark, description: this.lastSaid})
                this.action.step++;
            },
            2: async () => {
                // get labels of icons from Icons dictionary
                const typeLabels = Object.values(lmTypes).map(icon => icon.label);
                // check if response matches any labels
                const chosenType = typeLabels.filter(label => label.includes(this.lastSaid.toLowerCase()))[0];
                
                if (chosenType?.length > 0) {
                    const chosenTypeKey = parseInt(Object.keys(lmTypes).find(key => lmTypes[key].label == chosenType));
                    // get corrected icon id by getting index of chosen icon type in labels array created above
                    this.addLmStore.updatePendingLandmark({...this.addLmStore.pendingLandmark, landmark_type: chosenTypeKey, title: chosenType[0], voice: true})
                    this.action.step++;
                }
            },
        },
        'view': {},
        'near': {},
    } 

    constructor(store: MainStore) {
        this.addLmStore = store.addLm;
        makeAutoObservable(this);
    }

    private async processCurrentAction() {
        await this.actionHandlerMap[this.action.verb][this.action.step]();
        this.action = {...this.action, step: this.action.step + 1};   
    }

    startSpeech() {
        if (!this.panelVisible)
            this.panelVisible = true;

        this.setupSpokestack();
    }

    close = async () => {
        await Spokestack.deactivate();
        this.lastSaid = undefined
        this.partialSpeechResults = undefined
        this.feedback = undefined
        this.listening = false
        this.action = undefined;
        this.panelVisible = false
    }

    listen() {this.listening = true}
    stopListening() {this.listening = true}

    showPanel() {this.panelVisible = true};
    hidePanel() {this.panelVisible = false};

    toggleVisible(val: boolean) {
        this.panelVisible = val;
    }

    private setupSpokestack() {
        // Adds handler that will open the app, navigate to the Map screen and open the panel if necessary when Spokestack.activate() is called.
        // Also, listening will be set to true, and speechResult will be reset
        Spokestack.addEventListener('activate', async () => {
            console.log('[Voice]: Spokestack activated')
            this.listening = true
        });
        // Adds handler that will set listening to false when Spokestack.deactivate() is called
        Spokestack.addEventListener('deactivate', () => {
            if (this.action) 
                Spokestack.activate();
            else 
                this.listening = false
        })
        // Adds handler that will be called when Spokestack finishes recognizing some speech (once the user stops speaking).
        // Sets speechResult and previousSpeechResult to the processed result 
        Spokestack.addEventListener('recognize', ({transcript}) => {
            console.log('[Voice]: Spokestack finished recognizing')
            this.partialSpeechResults = undefined;
            this.lastSaid = transcript;
            this.previousResult = transcript;
            this.processSpeechTranscript()
        })
        // Adds handler that will be called when Spokestack recognizes an update to the speech it is currently listening to.
        // Sets partialSpeechResult to the processed result
        Spokestack.addEventListener('partial_recognize', ({transcript}) => {
            console.log('[Voice]: Spokestack recognizing')
            this.partialSpeechResults = transcript;
        })  

        Spokestack.addEventListener('error', ({error}) => {
            console.log('[Voice]: Spokestack error: ' + error)
        })

        // TODO: make these env variables
        Spokestack.initialize(
            'c361ff3a-70c3-42e6-b0ee-0207edd03b18', // account id
            '1A8196594C401EB93035CC6D7D6328CF1855C2B359744E720953AC34B6F658CA', // api token
            {
                pipeline: {
                    profile: PipelineProfile.PTT_NATIVE_ASR
                },
                // wakeword: {
                //     detect: 'https://s.spokestack.io/u/kHNUa/detect.tflite',
                //     filter: 'https://s.spokestack.io/u/kHNUa/filter.tflite',
                //     encode: 'https://s.spokestack.io/u/kHNUa/encode.tflite',
                // },
            }
        ).then(Spokestack.start).catch(error => console.log("[Voice]: Something went wrong when starting spokestack: " + error))

        return () => {
            Spokestack.removeAllListeners();
        };
    }

    async processSpeechTranscript () {
        // stop voice if speech result is one of the stop cases - highest priority
        if (this.shouldCloseVoice()) 
            await this.close();
        
        // when an action hasn't been initialized yet - discern which action is being requested
        else if (!this.action) {
            this.discernAction();
        }

        // when an action has been initialized and user says back - go back a step
        if (this.lastSaid.includes('back')) 
            this.action.step--

        // when an action has been initialized - perform the associated action flow
        else 
            await this.processCurrentAction();
    }

    /**
     * method that initates an appropiate action for the detected action request. it executes when there isn't an action set yet.
     * each initate<ActionName>Action() method will set the action, then setResponse to the user accordingly.
     * once this method has been called, all subsequent transcript will be piped into performActionFlow
     */
    private async discernAction() {
        // this branch will initiate whichever action is detected
        if (this.resultIncludesVerb('add') && this.lastSaid.includes("landmark")) {
            this.action = {verb: 'add', step: 1};
            return
        }
        // else if (this.resultIncludesVerb('near')) {
        //     this.action = {verb: 'near', step: 1}
        //     return
        // }
        // else if (speechResult.includes("filter landmarks")) {
        //     // example action
        // }
        // other possible actions

        // handle unrecognized result
        else {
            this.feedback = "Did not recognize command, please try again";
        }

        await Spokestack.activate();
    }

    private shouldCloseVoice() {
        return (
            STOP_RESPONSES.some(response => this.lastSaid.includes(response)) || 
            (this.lastSaid.includes('no') || this.lastSaid.includes('back')) && !this.action
        )
    }

    private resultIncludesVerb(verb: ActionVerb) { 
        return ACTION_VERB_MAP[verb].some(word => this.lastSaid.includes(word)); 
    }
}