﻿/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import { FontAwesome } from '@expo/vector-icons';
import { useNavigation } from '@react-navigation/native';
import React, { useEffect, useMemo, useState } from 'react';
import { ActivityIndicator, AppState, FlatList, ImageRequireSource, KeyboardAvoidingView, Linking, Text, TouchableOpacity, View, ViewStyle } from 'react-native';
import { Pulse, Wave } from 'react-native-animated-spinkit';
import FastImage from 'react-native-fast-image';
import Modal from "react-native-modal";
import { SafeAreaView } from 'react-native-safe-area-context';
import { MutationStatus } from 'react-query';
import { useAddLandmark } from '../../../../api/landmarks/landmark.add';
import { store } from '../../../../main-store';
import { colors, GlobalStyles, lmTypes } from '../../../../utils/GlobalUtils';
import { Separator } from '../../../Separator';
import { useVoicePanelApi } from './voice-panel.api';

// An array of words that will cause the app to stop listening once heard




/**
 * Panel that provides UI flows and backend logic for voice activated features. Relies on [Spokestack](https://www.npmjs.com/package/react-native-spokestack).
 * Check the documentation for more information about the Spokestack.initalize(), Spokestack.start(), and Spokestack.activate() and deactivate() methods.
 * A quick summary is as follows:
 * - Spokestack.initialize() -> Prepares the Spokestack service with a given configuration
 * - Spokestack.start() -> Starts the Spokestack service with the configuration given in initialize(). 
 *   After this method, Spokestack is ready to start recognizing speech, and can now be triggered by wakewords
 * - Spokestack.activate() -> Causes Spokestack to start listening and processing all speech and output the result. 
 *   This can be triggered manually or triggered by a wakeword 
 * - Spokestack.deactivate() -> Causes Spokestack to stop listening and processing all speech
 * - Spokestack.stop() -> After this method triggers, Spokestack no longer is able to listen for wakewords, or be activated
 * - Spokestack.destroy() -> The Spokestack service is torn down, and must be initialized with initialize() again for Spokestack to be used
 * @param 
 * @returns 
 */

 const ActionOptions: React.FC<{addLmStatus: MutationStatus}> = ({addLmStatus}) => {
    if (store.voice.action) {
        switch (store.voice.action.verb) {
            case 'add':
                switch (store.voice.action.step) {
                    case 1: 
                        return <Text style={{fontSize: 20, margin: 20, color: 'white'}}>Describe the landmark</Text> 
                    case 2:
                        return (
                            <View>
                                <Text style={{fontSize: 17, marginHorizontal: 20, marginBottom: 5, color: 'white'}}>Your description:</Text>
                                <Text style={{fontSize: 15, marginHorizontal: 20, marginBottom: 10, color:'white'}}>{store.addLm.pendingLandmark.description}</Text> 
                                <Text style={{fontSize: 17, margin: 20, color: 'white'}}>Choose from the available types: </Text> 
                                <FlatList 
                                    style={{marginBottom: 20, marginHorizontal:20}}
                                    data={Object.values(lmTypes).map(value => {return {label: value.label, image: value.image}})}
                                    numColumns={2}
                                    keyExtractor={(item) => item.label}
                                    renderItem={((item) => {
                                        return (
                                            <LmTypeDisplay key={item.item.label} lmType={item.item} style={{flexBasis: '50%'}} />
                                        )})} />
                            </View>
                        )
                    case 3:
                        return (
                            <View>
                                <Text style={{fontSize: 20, margin: 20, marginTop: 10, color: 'white'}}>Your landmark: </Text> 
                                <View style={{marginLeft: 40, marginBottom: 20}}>
                                    <Text style={{fontSize: 17, marginBottom: 10, color: 'white'}}>Description: </Text> 
                                    <Text style={{fontSize: 15, marginBottom: 20, marginLeft: 20, color: 'white'}}>{store.addLm.pendingLandmark.description}</Text> 
                                    <Text style={{fontSize: 17, marginRight: 10, marginBottom: 10, color: 'white'}}>Type: </Text>
                                    <LmTypeDisplay lmType={lmTypes[store.addLm.pendingLandmark.landmark_type]} style={{marginLeft: 20}} />
                                </View>
                                <Text style={{fontSize: 17, marginHorizontal: 20, marginBottom:10, color: 'white'}}>Are you sure you want to add this landmark?</Text> 
                                <Text style={{fontSize: 15, marginHorizontal: 40, marginBottom: 10, color: 'white'}}>- Say "Yes" to confirm</Text>
                                <Text style={{fontSize: 15, marginHorizontal: 40, marginBottom: 20, color: 'white'}}>- Say "No" to cancel</Text>
                            </View>
                        )
                    case 4:
                        return (
                            <View>
                                <Text style={{fontSize: 20, margin: 20, color: 'white'}}>Done. Anything else?</Text> 
                            </View>
                        )
                }
        }   
    }
    
    if (addLmStatus == 'loading') {
        return <ActivityIndicator color='white' size="large"/>
    }
    else if (addLmStatus == 'error') {
        return <Text style={{fontSize: 20, margin: 20, color: 'white'}}>Something went wrong when trying to upload the landmark.</Text>
    }
}

const LmTypeDisplay: React.FC<{lmType: {image: ImageRequireSource, label:string}, style?: ViewStyle}> = ({lmType, style}) => {
    return useMemo(() => (
        <View style={[{marginVertical: 5, flexDirection: 'row'}, style]}>  
            <FastImage style={{height: 25, width: 18}} source={lmType.image} />
            <Text style={{fontSize: 15, marginLeft: 10, textAlign: 'center', color: 'white'}}>{lmType.label}</Text>
        </View>
    ), [lmType])
}

const ActionProcessingIndicator: React.FC = () => {
    return (
        <View style={{margin: 20}}>
            <Text style={{fontSize: 20, margin: 20, color: 'white'}}>Adding landmark...</Text>
            <ActivityIndicator color="white" size="large" />
        </View>
    )
}

const SpeechActivity: React.FC = () => {
    return (
        <View style={{justifyContent: 'center', margin: 20, padding: 10, backgroundColor: 'white'}}>
            {store.voice.previousResult ? <Text style={{fontSize: 15, marginHorizontal: 20, marginBottom: 10, color: "black", alignSelf: 'center'}}>Last result: {store.voice.previousResult}</Text> : null }
            {store.voice.partialSpeechResults ?                         
            <Wave color={colors.red} hidesWhenStopped={false} animating={true} style={{alignSelf: 'center', marginBottom: 10}} /> : 
            <Pulse color={colors.red} hidesWhenStopped={false} animating={true} style={{alignSelf: 'center', marginBottom: 10}} />}
            <Text style={{fontSize: 15, marginHorizontal: 20, color: "black", alignSelf: 'center'}}>{
                store.voice.lastSaid ? store.voice.lastSaid :
                store.voice.partialSpeechResults ? store.voice.partialSpeechResults : ""}
            </Text>
        </View>
    )
}

const VoicePanelHeader: React.FC = () => {
    return (
        <View style={[GlobalStyles.itemRowContainer, {width: '100%'}]} >
            <Text style={{fontSize: 20, margin: 20, color: 'white', flexBasis: "70%"}}>{store.voice.action ? store.voice.action.verb : "Waiting for command..."}</Text>
            <TouchableOpacity style={{margin: 20}} onPress={async () => await store.voice.close()}><FontAwesome size={20} color='white' name='times'/></TouchableOpacity>
        </View>
    )
}

export const VoicePanel: React.FC = () => {
    const navigation = useNavigation();

    const {addLandmarkMutation} = useVoicePanelApi();

    // A string that represents a response that app gives to the user once it executes some voice recognition
    const [response, setResponse] = useState<string>();
    
    // Sets up and starts the Spokestack process the first time this component is rendered
    useEffect(() => {
        if (!navigation.isFocused()) {
            navigation.navigate('Map' as any)
        }
        if (AppState.currentState != 'active') {
            Linking.openURL("cnp.mobile://")
        }
    }, [store.voice.panelVisible])

    return (
    <Modal
        swipeDirection='down'
        useNativeDriver={true}
        useNativeDriverForBackdrop={true}
        avoidKeyboard={true}
        onBackdropPress={async () => await store.voice.close()}
        style={{justifyContent: "flex-end", height: '100%', margin: 0}}
        isVisible={store.voice.panelVisible} >
        <KeyboardAvoidingView>
            <SafeAreaView style={{backgroundColor: colors.red}}>
                {addLandmarkMutation.isLoading ?
                <ActionProcessingIndicator /> : 
                <View>
                    <VoicePanelHeader />
                    <Separator color="lightgray" style={{marginHorizontal: 20, marginBottom: 20, opacity: .5}} />
                    {store.voice.action ? 
                    <ActionOptions addLmStatus={addLandmarkMutation.status}/> : 
                    <View style={{marginBottom: 20}}>
                        <Text style={{fontSize: 15, marginHorizontal: 40, marginBottom: 10, color: 'white'}}>- Say "Add landmark" to add a landmark</Text>
                        <Text style={{fontSize: 15, marginHorizontal: 40, marginBottom: 10, color: 'white'}}>- Say "Near" to view nearby landmarks</Text>
                    </View>}
                    {response? <Text style={{fontSize: 13, marginHorizontal: 40, marginBottom: 20, color: 'white'}}>{response}</Text> : null}
                    <SpeechActivity />
                    <Text style={{fontSize: 13, marginHorizontal: 40, marginBottom: 20, color: 'white'}}>Say "cancel", "bye", "goodbye", or "stop" to exit.</Text>
                    <Text style={{fontSize: 13, marginHorizontal: 40, marginBottom: 20, color: 'white'}}>Say "back" to go back a step.</Text>
                </View>}
            </SafeAreaView>
        </KeyboardAvoidingView>
    </Modal>
    )
}